@@ -12,11 +12,12 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.!
 
+#include "builder.h"
+
 #include <algorithm>
 #include <functional>
 #include <utility>
 
-#include "builder.h"
 #include "filesystem.h"
 #include "third_party/absl/strings/str_join.h"
 #include "third_party/absl/strings/str_replace.h"
@@ -218,8 +219,9 @@ util::Status Builder::DecompileCharsMap(absl::string_view blob,
   chars_map->clear();
 
   absl::string_view trie_blob, normalized;
-  RETURN_IF_ERROR(
-      Normalizer::DecodePrecompiledCharsMap(blob, &trie_blob, &normalized));
+  std::string buf;
+  RETURN_IF_ERROR(Normalizer::DecodePrecompiledCharsMap(blob, &trie_blob,
+                                                        &normalized, &buf));
 
   Darts::DoubleArray trie;
   trie.set_array(const_cast<char *>(trie_blob.data()),
@@ -476,7 +478,7 @@ util::Status Builder::BuildNmtNFKC_CFMap(CharsMap *chars_map) {
 // static
 util::Status Builder::LoadCharsMap(absl::string_view filename,
                                    CharsMap *chars_map) {
-  LOG(INFO) << "Loading maping file: " << filename.data();
+  LOG(INFO) << "Loading mapping file: " << filename.data();
   CHECK_OR_RETURN(chars_map);
 
   auto input = filesystem::NewReadableFile(filename);
@@ -487,16 +489,16 @@ util::Status Builder::LoadCharsMap(absl::string_view filename,
   chars_map->clear();
   while (input->ReadLine(&line)) {
     std::vector<std::string> fields =
-        absl::StrSplit(line, "\t", absl::AllowEmpty());
+        absl::StrSplit(line, '\t', absl::AllowEmpty());
     CHECK_GE(fields.size(), 1);
     if (fields.size() == 1) fields.push_back("");  // Deletion rule.
     std::vector<char32> src, trg;
-    for (auto s : absl::StrSplit(fields[0], " ")) {
+    for (auto s : absl::StrSplit(fields[0], ' ')) {
       if (s.empty()) continue;
       absl::ConsumePrefix(&s, "U+");
       src.push_back(string_util::HexToInt<char32>(s));
     }
-    for (auto s : absl::StrSplit(fields[1], " ")) {
+    for (auto s : absl::StrSplit(fields[1], ' ')) {
       if (s.empty()) continue;
       absl::ConsumePrefix(&s, "U+");
       trg.push_back(string_util::HexToInt<char32>(s));
