@@ -45,6 +45,9 @@ const char kSpaceSymbol[] = "\xe2\x96\x81";
 // since this character can be useful both for user and
 // developer. We can easily figure out that <unk> is emitted.
 const char kDefaultUnknownSymbol[] = " \xE2\x81\x87 ";
+
+// REPLACEMENT CHARACTER (U+FFFD) in UTF-8.
+const char kReplacementCharacter[] = "\xef\xbf\xbd";
 }  // namespace
 
 SentencePieceProcessor::SentencePieceProcessor() {}
@@ -78,7 +81,6 @@ util::Status SentencePieceProcessor::Load(
     std::unique_ptr<ModelProto> model_proto) {
   model_proto_ = std::move(model_proto);
   model_ = ModelFactory::Create(*model_proto_);
-
   normalizer_ = absl::make_unique<normalizer::Normalizer>(
       model_proto_->normalizer_spec(), model_proto_->trainer_spec());
   if (model_proto_->has_denormalizer_spec() &&
@@ -544,49 +546,67 @@ util::Status SentencePieceProcessor::Decode(
   RETURN_IF_ERROR(ApplyExtraOptions(decode_extra_options_, spt));
 
   std::string *text = spt->mutable_text();
-  auto SetSurface = [&](int index, const std::string &surface) {
+  auto SetSurface = [&](int index, absl::string_view surface) {
     auto *sp = spt->mutable_pieces(index);
-    sp->set_surface(surface);
+    sp->set_surface(std::string(surface));
     sp->set_begin(text->size());
     sp->set_end(text->size() + surface.size());
-    *text += surface;
+    absl::StrAppend(text, surface);
   };
-  auto ProcessBytePieces = [&](int begin, int end) -> util::Status {
-    if (begin < end) {
-      // Constructs byte sequence.
-      std::string bytes;
-      for (int i = begin; i < end; ++i) {
-        const auto &sp = spt->pieces(i);
-        const int byte = PieceToByte(sp.piece());
-        CHECK_LE_OR_RETURN(0, byte);
-        bytes.append(1, byte);
-      }
-      // Decodes byte sequence as UTF-8 and encodes the result into UTF-8 bytes
-      // again.
-      int i = begin;
-      for (const char32 uc :
-           string_util::UTF8ToUnicodeText(absl::string_view(bytes))) {
-        if (uc == kUnicodeError) {
-          // Invalid UTF-8 bytes are mapped to REPLACEMENT CHARACTER (U+FFFD).
-          SetSurface(i++, string_util::UnicodeCharToUTF8(kUnicodeError));
-        } else {
-          const std::string utf8 = string_util::UnicodeCharToUTF8(uc);
-          for (int j = 0; j < utf8.size(); j++) {
-            // The last byte piece holds the surface of the original unknown
-            // character. The other byte pieces hold an empty string as
-            // surface.
-            if (j == utf8.size() - 1) {
-              SetSurface(i++, utf8);
-            } else {
-              SetSurface(i++, "");
-            }
+
+  auto ProcessBytePieces = [&](int token_index_begin,
+                               int token_index_end) -> util::Status {
+    if (token_index_begin >= token_index_end) {
+      return util::OkStatus();
+    }
+
+    // Constructs byte sequence.
+    std::string bytes;
+    for (int i = token_index_begin; i < token_index_end; ++i) {
+      const auto &sp = spt->pieces(i);
+      const int byte = PieceToByte(sp.piece());
+      CHECK_LE_OR_RETURN(0, byte);
+      bytes.append(1, byte);
+    }
+
+    // Set surfaces of `bytes` for each Unicode character.
+    int offset = 0;
+    const int bytes_len = bytes.size();
+    while (offset < bytes_len) {
+      // Consume `bytes` by one Unicode character.
+      size_t consumed;  // Number of bytes consumed in this iteration.
+      const bool is_valid = string_util::IsValidDecodeUTF8(
+          absl::string_view(bytes).substr(offset), &consumed);
+
+      // Set surfaces of the consumed byte pieces.
+      const int token_index = token_index_begin + offset;
+
+      if (!is_valid) {
+        // The byte piece at `token_index` is structurally invalid. Map it to
+        // REPLACEMENT CHARACTER (U+FFFD).
+        CHECK_EQ_OR_RETURN(consumed, 1);
+        SetSurface(token_index, kReplacementCharacter);
+      } else {
+        const absl::string_view utf8 =
+            absl::string_view(bytes).substr(offset, consumed);
+        for (int j = 0; j < consumed; j++) {
+          // The last byte piece holds the surface of the original unknown
+          // character. The other byte pieces hold an empty string as
+          // surface.
+          if (j == consumed - 1) {
+            SetSurface(token_index + j, utf8);
+          } else {
+            SetSurface(token_index + j, "");
           }
         }
       }
-      CHECK_EQ_OR_RETURN(i, end);
+      offset += consumed;
     }
+    CHECK_EQ_OR_RETURN(token_index_begin + offset, token_index_end);
+
     return util::OkStatus();
   };
+
   int byte_start = 0;
   for (int i = 0; i < spt->pieces_size(); ++i) {
     const auto &sp = spt->pieces(i);
@@ -608,8 +628,13 @@ util::Status SentencePieceProcessor::Decode(
 util::Status SentencePieceProcessor::Decode(const std::vector<int> &ids,
                                             SentencePieceText *spt) const {
   std::vector<std::string> pieces;
+  const int num_pieces = GetPieceSize();
   pieces.reserve(ids.size());
   for (const int id : ids) {
+    if (id < 0 || id >= num_pieces) {
+      return util::Status(util::StatusCode::kOutOfRange,
+                          absl::StrCat("Invalid id: ", id));
+    }
     pieces.emplace_back(IdToPiece(id));
   }
   return Decode(pieces, spt);
