@@ -12,8 +12,6 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.!
 
-#include "unigram_model.h"
-
 #include <algorithm>
 #include <cfloat>
 #include <cmath>
@@ -23,7 +21,10 @@
 #include <utility>
 #include <vector>
 
+#include "third_party/absl/memory/memory.h"
+#include "third_party/absl/strings/str_split.h"
 #include "third_party/absl/strings/string_view.h"
+#include "unigram_model.h"
 #include "util.h"
 
 namespace sentencepiece {
@@ -33,6 +34,9 @@ namespace {
 // Size of nodes pre-allocated in Lattice.
 constexpr size_t kPreallocateLatticeNodeSize = 1024;
 
+constexpr float kUnkPenalty = 10.0;
+constexpr float kEpsilon = 1e-7;
+
 // Returns log(exp(x) + exp(y)).
 // if init_mode is true, returns log(exp(y)) == y.
 // log(\sum_i exp(a[i])) can be computed as
@@ -48,7 +52,7 @@ inline float LogSumExp(float x, float y, bool init_mode) {
   if (vmax > vmin + kMinusLogEpsilon) {
     return vmax;
   } else {
-    return vmax + log(exp(vmin - vmax) + 1.0);
+    return vmax + log(std::exp(static_cast<double>(vmin - vmax)) + 1.0);
   }
 }
 }  // namespace
@@ -213,8 +217,10 @@ float Lattice::PopulateMarginal(float freq,
     for (Node *node : begin_nodes_[pos]) {
       if (node->id >= 0) {
         // the index of |expected| is a Node::id, which is a vocabulary id.
-        (*expected)[node->id] += freq * exp(alpha[node->node_id] + node->score +
-                                            beta[node->node_id] - Z);
+        (*expected)[node->id] +=
+            freq *
+            std::exp(static_cast<double>(alpha[node->node_id] + node->score +
+                                         beta[node->node_id] - Z));
       }
     }
   }
@@ -349,7 +355,8 @@ std::vector<Lattice::Node *> Lattice::Sample(float theta) {
   while (true) {
     probs.clear();
     for (const Node *lnode : end_nodes_[node->pos]) {
-      probs.push_back(exp(alpha[lnode->node_id] + theta * lnode->score - Z));
+      probs.push_back(std::exp(static_cast<double>(alpha[lnode->node_id] +
+                                                   theta * lnode->score - Z)));
     }
     std::discrete_distribution<int> dist(probs.begin(), probs.end());
     node = end_nodes_[node->pos][dist(*mt)];
@@ -373,7 +380,6 @@ void Model::PopulateNodes(Lattice *lattice) const {
     return pos - begin_pos;
   };
 
-  constexpr float kUnkPenalty = 10.0;
   const float unk_score = min_score() - kUnkPenalty;
 
   const int len = lattice->size();
@@ -403,7 +409,7 @@ void Model::PopulateNodes(Lattice *lattice) const {
       Lattice::Node *node = lattice->Insert(begin_pos, length);
       node->id = id;  // the value of Trie stores vocab_id.
       // User defined symbol receives extra bonus to always be selected.
-      node->score = IsUserDefinedInlined(id) ? (length * max_score_ + 1.0)
+      node->score = IsUserDefinedInlined(id) ? (length * max_score_ - 0.1)
                                              : GetScoreInlined(id);
       if (!has_single_node && node->length == 1) {
         has_single_node = true;
@@ -424,7 +430,7 @@ int Model::PieceToId(absl::string_view piece) const {
     return it->second;
   }
   int id = 0;
-  trie_->exactMatchSearch(piece.data(), id);
+  trie_->exactMatchSearch(piece.data(), id, piece.size());
   return id == -1 ? unk_id_ : id;
 }
 
@@ -448,7 +454,7 @@ void Model::BuildTrie(std::vector<std::pair<absl::string_view, int>> *pieces) {
     value[i] = (*pieces)[i].second;      // vocab_id
   }
 
-  trie_ = port::MakeUnique<Darts::DoubleArray>();
+  trie_ = absl::make_unique<Darts::DoubleArray>();
   if (trie_->build(key.size(), const_cast<char **>(&key[0]), nullptr,
                    &value[0]) != 0) {
     status_ = util::InternalError("cannot build double-array.");
@@ -495,6 +501,10 @@ Model::Model(const ModelProto &model_proto) {
 Model::~Model() {}
 
 EncodeResult Model::Encode(absl::string_view normalized) const {
+  if (encoder_version_ == EncoderVersion::kOptimized) {
+    return EncodeOptimized(normalized);
+  }
+
   if (!status().ok() || normalized.empty()) {
     return {};
   }
@@ -555,5 +565,169 @@ EncodeResult Model::SampleEncode(absl::string_view normalized,
   return results;
 }
 
+bool Model::VerifyOutputsEquivalent(absl::string_view expected,
+                                    absl::string_view actual) const {
+  auto compute_unigram_model_score =
+      [this](std::vector<absl::string_view> output_pieces) {
+        float total_score = 0;
+        const float unk_score = min_score() - kUnkPenalty;
+        for (const auto p : output_pieces) {
+          const auto id = PieceToId(p);
+          if (id == unk_id_) {
+            total_score += unk_score;
+          } else {
+            const int length = p.size();
+            total_score += IsUserDefinedInlined(id)
+                               ? (length * max_score_ - 0.1)
+                               : GetScoreInlined(id);
+          }
+        }
+        return total_score;
+      };
+  const auto expected_score =
+      compute_unigram_model_score(absl::StrSplit(expected, ' '));
+  const auto actual_score =
+      compute_unigram_model_score(absl::StrSplit(actual, ' '));
+  if (std::abs(expected_score - actual_score) > kEpsilon) {
+    LOG(WARNING) << "Two sentence piece sequences are not equivalent! Left: "
+                 << expected << ", Score: " << expected_score
+                 << ". Right: " << actual << ", Score: " << actual_score << ".";
+    return false;
+  }
+  return true;
+}
+
+EncodeResult Model::EncodeOptimized(absl::string_view normalized) const {
+  // An optimized Viterbi algorithm for unigram language models. Benchmarking
+  // results show that it generates almost identical outputs and achieves 2.1x
+  // speedup on average for 102 languages compared to the original
+  // implementation. It's based on the following three ideas:
+  //
+  // 1. Because it uses the *unigram* model:
+  //     best_score(x1, x2, …, xt) = best_score(x1, x2, …, x{t-1}) + score(xt)
+  // Deciding the best path (and score) can be decoupled into two isolated
+  // terms: (a) the best path ended before the last token `best_score(x1, x2, …,
+  // x{t-1})`, and (b) the last token and its `score(xt)`. The two terms are
+  // not related to each other at all.
+  //
+  // Therefore, we can compute once and store the *best_path ending at
+  // each character position*. In this way, when we know best_path_ends_at[M],
+  // we can reuse it to compute all the best_path_ends_at_[...] where the last
+  // token starts at the same character position M.
+  //
+  // This improves the time complexity from O(n*k*k) to O(n*k) because it
+  // eliminates the extra loop of recomputing the best path ending at the same
+  // position, where n is the input length and k is the maximum number of tokens
+  // that can be recognized starting at each position.
+  //
+  // 2. Again, because it uses the *unigram* model, we don’t need to actually
+  // store the lattice nodes. We still recognize all the tokens and lattice
+  // nodes from the input, but along identifying them, we use and discard them
+  // on the fly. There is no need to actually store them for best path Viterbi
+  // decoding. The only thing we need to store is the best_path ending at
+  // each character position.
+  //
+  // This improvement reduces the things needed to store in memory from O(n*k)
+  // to O(n), where n is the input length and k is the maximum number of tokens
+  // that can be recognized starting at each position.
+  //
+  // It also avoids the need of dynamic-size lattice node pool, because the
+  // number of things to store is fixed as n.
+  //
+  // 3. SentencePiece is designed to work with unicode, taking utf-8 encoding
+  // inputs. In the original implementation, the lattice positions are based on
+  // unicode positions. A mapping from unicode position to the utf-8 position is
+  // maintained to recover the utf-8 string piece.
+  //
+  // We found that it is sufficient and beneficial to directly work with utf-8
+  // positions:
+  //
+  // Firstly, it saves the conversion and mapping between unicode positions and
+  // utf-8 positions.
+  //
+  // Secondly, it reduces the number of fields we need to maintain in the
+  // node/path structure. Specifically, there are 8 fields defined in
+  // `Lattice::Node` used by the original encoder, but here in the optimized
+  // encoder we only need to define 3 fields in `BestPathNode`.
+
+  if (!status().ok() || normalized.empty()) {
+    return {};
+  }
+  // Represents the last node of the best path.
+  struct BestPathNode {
+    int id = -1;  // The vocab id. (maybe -1 for UNK)
+    float best_path_score =
+        0;  // The total score of the best path ending at this node.
+    int starts_at =
+        -1;  // The starting position (in utf-8) of this node. The entire best
+             // path can be constructed by backtracking along this link.
+  };
+  const int size = normalized.size();
+  const float unk_score = min_score() - kUnkPenalty;
+  // The ends are exclusive.
+  std::vector<BestPathNode> best_path_ends_at(size + 1);
+  // Generate lattice on-the-fly (not stored) and update best_path_ends_at.
+  int starts_at = 0;
+  while (starts_at < size) {
+    std::size_t node_pos = 0;
+    std::size_t key_pos = starts_at;
+    const auto best_path_score_till_here =
+        best_path_ends_at[starts_at].best_path_score;
+    bool has_single_node = false;
+    const int mblen =
+        std::min<int>(string_util::OneCharLen(normalized.data() + starts_at),
+                      size - starts_at);
+    while (key_pos < size) {
+      const int ret =
+          trie_->traverse(normalized.data(), node_pos, key_pos, key_pos + 1);
+      if (ret == -2) break;
+      if (ret >= 0) {
+        if (IsUnusedInlined(ret)) continue;
+        // Update the best path node.
+        auto &target_node = best_path_ends_at[key_pos];
+        const auto length = (key_pos - starts_at);
+        // User defined symbol receives extra bonus to always be selected.
+        const auto score = IsUserDefinedInlined(ret)
+                               ? (length * max_score_ - 0.1)
+                               : GetScoreInlined(ret);
+        const auto candidate_best_path_score =
+            score + best_path_score_till_here;
+        if (target_node.starts_at == -1 ||
+            candidate_best_path_score > target_node.best_path_score) {
+          target_node.best_path_score = candidate_best_path_score;
+          target_node.starts_at = starts_at;
+          target_node.id = ret;
+        }
+        if (!has_single_node && length == mblen) {
+          has_single_node = true;
+        }
+      }
+    }
+    if (!has_single_node) {
+      auto &target_node = best_path_ends_at[starts_at + mblen];
+      const auto candidate_best_path_score =
+          unk_score + best_path_score_till_here;
+      if (target_node.starts_at == -1 ||
+          candidate_best_path_score > target_node.best_path_score) {
+        target_node.best_path_score = candidate_best_path_score;
+        target_node.starts_at = starts_at;
+        target_node.id = unk_id_;
+      }
+    }
+    // Move by one unicode character.
+    starts_at += mblen;
+  }
+  // Backtrack to identify the best path.
+  EncodeResult results;
+  int ends_at = size;
+  while (ends_at > 0) {
+    const auto &node = best_path_ends_at[ends_at];
+    results.emplace_back(
+        normalized.substr(node.starts_at, ends_at - node.starts_at), node.id);
+    ends_at = node.starts_at;
+  }
+  std::reverse(results.begin(), results.end());
+  return results;
+}
 }  // namespace unigram
 }  // namespace sentencepiece
