@@ -20,6 +20,7 @@ namespace sentencepiece {
 namespace {
 constexpr unsigned int kDefaultSeed = static_cast<unsigned int>(-1);
 static unsigned int g_seed = kDefaultSeed;
+static int g_minloglevel = 0;
 }  // namespace
 
 void SetRandomGeneratorSeed(unsigned int seed) {
@@ -30,6 +31,11 @@ uint32 GetRandomGeneratorSeed() {
   return g_seed == kDefaultSeed ? std::random_device{}() : g_seed;
 }
 
+namespace logging {
+int GetMinLogLevel() { return g_minloglevel; }
+void SetMinLogLevel(int v) { g_minloglevel = v; }
+}  // namespace logging
+
 namespace string_util {
 
 // mblen sotres the number of bytes consumed after decoding.
